
Bullet = {}
Bullet.__index = Bullet

function Bullet.new(hurtTo, x, y, angle, speed, damage, continuous)
    local self = Thing.new({'bullet'}):extend(Bullet)

    self.hurtTo = hurtTo
    self.x, self.y = x, y
    self.r = 5
    self.damage = damage
    self.angle = angle
    self.speed = speed
    self.continuous = continuous

    if self.continuous then
        self.lifeTime = 0.3
    else
        self.lifeTime = 5
    end
    self.remainTime = self.lifeTime

    return self
end

function Bullet.getAll()
    return Thing.getAll('bullet')
end

function Bullet:draw()
    g.push()
    g.translate(self.x, self.y)
    g.scale(self.r/content.images.bullet_r)
    g.rotate(self.angle)
    if self.hurtTo == 'enemy' then
        g.setColor(255, 255, 255, 255*self.remainTime/self.lifeTime)
    elseif self.hurtTo == 'core' then
        g.setColor(255, 0, 150, 255*self.remainTime/self.lifeTime)
    end
    g.draw(content.images.bullet, -content.images.bullet_hw, -content.images.bullet_hh)
    g.pop()
end

function Bullet:always()
    if self:alive() then
        self:processMove()
        self:processCollide()
        self:processOut()
    end
end

function Bullet:processMove()
    self.x = self.x + math.cos(self.angle)*self.speed*dt
    self.y = self.y + math.sin(self.angle)*self.speed*dt
end

function Bullet:processCollide()
    local list = Thing.getAll(self.hurtTo)

    for i, t in ipairs(list) do
        local dist = math.sqrt(math.pow(self.x - t.x, 2) + math.pow(self.y - t.y, 2))

        if dist <= t.r + self.r then
            if t.performHurt then
                t:performHurt(self.damage)
                self:doScore(t)
            elseif t.performDestroy then
                t:performDestroy(true, self.continuous)
                self:doScore(t)
            end

            self:performDestroy()
            break
        end
    end
end

function Bullet:processOut()
    self.remainTime = self.remainTime - dt
    if self.remainTime <= 0 or self.x < 0 - self.r  or self.x > g.getWidth() - self.r or
       self.y < 0 - self.r or self.y > g.getHeight() - self.r then
        self:destroy()
    end
end

function Bullet:performDestroy()
    self:destroy()
end

function Bullet:doScore(t)
    if t.tags:contain('enemy') then
        Score.instance():add(t.damage)
    end
end