require 'flatmoon.tweener'

Collector = {}
Collector.__index = Collector

local instance = nil

function Collector.new()
    --if instance and instance:alive() then return instance end

    instance = Thing.new({'collector'}):extend(Collector)

    instance.x, instance.y = love.mouse.getX(), love.mouse.getY()
    instance.r = 25

    instance.originR = instance.r

    instance.isActive = true

    instance.icon = {img = content.images.collectoricon, hw = content.images.collectoricon_hw, hh = content.images.collectoricon_hh}

    return instance
end

function Collector.instance()
    assert(instance, 'instance expects new before')

    return instance
end

function Collector:draw()
    g.push()
    g.translate(self.x, self.y)
    g.scale(self.r/content.images.collector_r)
    g.setColor(0, 100, 250, 200)
    g.draw(content.images.collector, -content.images.collector_hw, -content.images.collector_hh)
    g.pop()
end


function Collector:always()
    if self:alive() then
        self.x, self.y = love.mouse.getX(), love.mouse.getY()

        self:processItem()
    end
end

function Collector:processItem()
    local list = Thing.getAll('item')

    for i, it in ipairs(list) do
        local dist = math.sqrt(math.pow(self.x - it.x, 2) + math.pow(self.y - it.y, 2))

        if dist <= math.abs(self.r - it.r) then
            it:active()

            --effecting
            love.audio.newSource(content.sounds.collected):play()
            self:seq(function ()
                tweener.tween(self, 0.2, {r = self.r/2})
                tweener.tween(self, 0.2, {r = self.originR})
            end)
        end
    end
end

function Collector:active()
    self.isActive = true
    self:setUpdate(true)
    self:setDraw(true)
end

function Collector:deactive()
    self.isActive = false
    self:setUpdate(false)
    self:setDraw(false)
end