
Cover = {}
Cover.__index = Cover

local instance = nil

function Cover.new()
    if instance and instance:alive() then return instance end

    instance = Thing.new({'cover'}):extend(Cover)

    instance.x, instance.y = Core.instance().x, Core.instance().y
    instance.angle = 0
    instance.r = 50
    instance.opposite = false

    instance.hurt = false

    instance.isActive = true

    instance.icon = {img = content.images.covericon, hw = content.images.covericon_hw, hh = content.images.covericon_hh}

    return instance
end

function Cover.instance()
    assert(instance, 'instance expects new before')

    return instance
end

function Cover.newnew(newclass, ...)
    local old = instance
    instance = nil
    local result = Cover.new(...)
    instance = old

    return result
end

function Cover:always()
    local core = Core.instance()
    local mx, my = love.mouse.getX(), love.mouse.getY()
    self.x, self.y = core.x, core.y
    self.angle = (self.opposite and math.pi or 0) + math.atan2(my - self.y, mx - self.x)

    self:processEnemy()
    self:processBullet()
end

function Cover:draw()
    g.push()
    g.translate(self.x, self.y)
    g.rotate(self.angle)
    g.scale(self.r/content.images.cover_r)
    if self.hurt then
        g.setColor(150, 100, 250, 200)
    else
        g.setColor(0, 100, 250, 200)
    end
    g.draw(content.images.cover, -content.images.cover_hw, -content.images.cover_hh)
    g.pop()
end

function Cover:processEnemy()
    local list = Enemy.getAll()

    local bold = content.images.cover_bold
    local ratio = self.r/content.images.cover_r
    local lineWidth = bold*ratio

    for i, e in ipairs(list) do
        local dist = math.sqrt(math.pow(self.x - e.x, 2) + math.pow(self.y - e.y, 2))

        if dist <= e.r + self.r and
           dist >= self.r - e.r - lineWidth and
           math.abs(self.angle - e.angle) > math.pi/2 then
            self:killEnemy(e)

            --effecting
            self:seq(function ()
                self.hurt = true
                wait(0.2)
                self.hurt = false
            end)
        end
    end
end

function Cover:processBullet()
    local list = Bullet.getAll()

    local bold = content.images.cover_bold
    local ratio = self.r/content.images.cover_r
    local lineWidth = bold*ratio

    for i, b in ipairs(list) do
        if b.hurtTo == 'core' then
            local dist = math.sqrt(math.pow(self.x - b.x, 2) + math.pow(self.y - b.y, 2))

            if dist <= b.r + self.r and
               dist >= self.r - b.r - lineWidth and
               math.abs(self.angle - b.angle) > math.pi/2 then
                self:killBullet(b)

                --effecting
                self:seq(function ()
                    self.hurt = true
                    wait(0.2)
                    self.hurt = false
                end)
            end
        end
    end
end

function Cover:killEnemy(e)
    --effecting
    love.audio.newSource(content.sounds.covercollide):play()

    e:performDestroy()
end

function Cover:killBullet(b)
    --effecting
    love.audio.newSource(content.sounds.covercollide):play()

    b:performDestroy()
end

function Cover:active()
    self.isActive = true

    self:setUpdate(true)
    self:setDraw(true)
end

function Cover:deactive()
    self.isActive = false

    self:setUpdate(false)
    self:setDraw(false)
end