require 'flatmoon.tweener'

require 'covere.scenes.play.Cover'
require 'covere.scenes.play.Gun'
require 'covere.scenes.play.Collector'

require 'covere.scenes.play.Item'

Defend = {}
Defend.__index = Defend

local instance = nil

function Defend.new()
    if instance and instance:alive() then return instance end

    instance = Thing.new({'defend'}):extend(Defend)

    instance.list = {}
    instance.effect = {}

    listener.add('keypressed', '', function (key) instance:onKey(key) end)

    return instance
end

function Defend.instance()
    assert(instance, 'instance expects new before')

    return instance
end

function Defend:draw()
    g.setColor(255, 255, 255)

    local cy = 640 
    for i, v in ipairs(self.list) do
        cy = cy - (10 + v.icon.hh*2*self.effect[i])

        g.push()
        g.translate(10, cy)
        g.scale(self.effect[i])
        g.draw(v.icon.img, 0, 0)
        g.pop()
    end
end

function Defend:add(kind)
    local a = kind.new()
    a:deactive()
    table.insert(self.list, a)
    table.insert(self.effect, 0.5)
end

function Defend:choose(index)
    if self.list[index].isActive then return end

    for i, v in ipairs(self.list) do
        if v.isActive then
            v:deactive()

            --effecting
            self:seq(function ()
                tweener.tween(self.effect, 0.5, {[i] = 0.5})
            end)
        end
    end

    self.list[index]:active()

    --effecting
    love.audio.newSource(content.sounds.switch):play()
    self:seq(function ()
        tweener.tween(self.effect, 0.5, {[index] = 1})
    end)
end

function Defend:chooseEx(...)
    local arg = {...}
    local ok = {}
    for i, v in ipairs(arg) do
        for j, u in ipairs(self.list) do
            if getmetatable(u) == v then
                ok[j] = true
            end
        end
    end

    for i, v in ipairs(self.list) do
        if ok[i] and not v.isActive then
            v:active()

            --effecting
            love.audio.newSource(content.sounds.switch):play()
            self:seq(function ()
                tweener.tween(self.effect, 0.5, {[i] = 1})
            end)
        elseif not ok[i] and v.isActive then
            v:deactive()

            --effecting
            self:seq(function ()
                tweener.tween(self.effect, 0.5, {[i] = 0.5})
            end)
        end
    end
end

function Defend:onKey(key)
    key = tonumber(key)
    if key and self.list[key] then
        self:choose(key)
    end
end