require 'flatmoon.tweener'

require 'covere.scenes.play.BulletStorm'

Enemy = {}
Enemy.__index = Enemy

function Enemy.new(x, y, speed)
    local self = Thing.new({'enemy'}):extend(Enemy)

    self.x, self.y = x, y
    self.r = 10
    self.damage = 1
    self.angle = 0
    self.speed = speed

    return self
end

function Enemy.getAll()
    return Thing.getAll('enemy')
end

function Enemy:always()
    if self:alive() then
        self:processMove()
        self:processCore()
    end
end

function Enemy:draw()
    g.push()
    g.translate(self.x, self.y)
    g.scale(self.r/content.images.enemy_r)
    g.rotate(self.angle)
    g.setColor(255, 50, 50, 255)
    g.draw(content.images.enemy, -content.images.enemy_hw, -content.images.enemy_hh)
    g.pop()
end

function Enemy:processMove()
    self.angle = math.atan2(Core.instance().y - self.y, Core.instance().x - self.x)
    self.x = self.x + math.cos(self.angle)*self.speed*dt
    self.y = self.y + math.sin(self.angle)*self.speed*dt
end

function Enemy:processCore()
    local core = Core.instance()
    local cx, cy, cr = core.x, core.y, core.r
    local dist = math.sqrt(math.pow(self.x - cx, 2) + math.pow(self.y - cy, 2))

    if dist <= self.r + cr then
        self:hurtCore()
    end
end

function Enemy:hurtCore()
    Core.instance():performHurt(self.damage)
    self:performDestroy()
end

function Enemy:performDestroy(explode, continuous)
    self:markAsDestroyed()

    --effecting
    love.audio.newSource(content.sounds.enemydown):play()
    self:seq(function ()
        self.speed = 0
        tweener.tween(self, 0.5, {r = 0}, nil, 'bounce')
        if explode then self:explode(continuous) end
        self:destroy()
    end)
end

function Enemy:explode(continuous)
    local bs = BulletStorm.new(Bullet, self.x, self.y, self.angle)
    if continuous then
        bs:fire()
    end
end