require 'flatmoon.tweener'

Gun = {}
Gun.__index = Gun

local instance = nil

function Gun.new()
    if instance and instance:alive() then return instance end

    instance = Cover.newnew():extend(Gun)

    instance.tags:add('gun')

    instance.fireSpeed = 500
    instance.originR = instance.r

    instance.lis1 = nil

    instance.icon = {img = content.images.gunicon, hw = content.images.gunicon_hw, hh = content.images.gunicon_hh}

    return instance
end

function Gun.instance()
    assert(instance, 'instance expects new before')

    return instance
end

function Gun:draw()
    g.push()
    g.translate(self.x, self.y)
    g.scale(self.r/content.images.gun_r)
    g.rotate(self.angle)
    g.setColor(0, 100, 250, 200)
    g.draw(content.images.gun, -content.images.gun_hw, -content.images.gun_hh)
    g.pop()
end

function Gun:processEnemy()
    --overriden
    --do nothing
end

function Gun:processBullet()
    --overriden
    --do nothing
end

function Gun:fire()
    local fx, fy = self.x + self.r*math.cos(self.angle), self.y + self.r*math.sin(self.angle)
    Bullet.new('enemy', fx, fy, self.angle, self.fireSpeed, 1, false)

    --effecting
    love.audio.newSource(content.sounds.gunfire):play()
    self:seq(function ()
        tweener.tween(self, 0.1, {r = self.r - 5})
        tweener.tween(self, 0.1, {r = self.originR})
    end)
end

function Gun:active()
    Cover.active(self)

    self.lis1 = listener.add('mousepressed', 'l', function () self:fire() end)
end

function Gun:deactive()
    Cover.deactive(self)

    if self.lis1 then
        listener.remove2(self.lis1)
        self.lis1 = nil
    end
end