require 'flatmoon.tweener'

require 'covere.scenes.play.Enemy'
require 'covere.scenes.play.Soldier'

require 'covere.scenes.play.Item'
require 'covere.scenes.play.AutoStorm'
require 'covere.scenes.play.TripleShot'
require 'covere.scenes.play.CombineAhead'
require 'covere.scenes.play.CombineBack'
require 'covere.scenes.play.Recover'
require 'covere.scenes.play.Bomber'

Spawner = {}
Spawner.__index = Spawner

local instance = nil

function Spawner.new()
    if instance and instance:alive() then return instance end

    instance = Thing.new({'spawner'}):extend(Spawner)

    instance.distance = 100
    instance.danger = {}

    return instance
end

function Spawner.instance()
    assert(instance, 'instance expects new before')

    return instance
end

function Spawner:draw()
    for i, d in ipairs(self.danger) do
        if d ~= 0 then
            g.setColor(255, 0, 0, 255*d.ratio)
            g.draw(content.images.danger, d.x - content.images.danger_hw, d.y - content.images.danger_hh)
        end
    end
end

function Spawner:addEnemy(kind, amount, minSpeed, maxSpeed, group, fromAngle, deltaAngle)
    amount = amount or 1
    minSpeed = minSpeed or 20
    maxSpeed = maxSpeed or 50
    if group == nil then group = false end

    local cx = Core.instance().x
    local cy = Core.instance().y
    local dx, dy = 0, 0

    local startAngle = fromAngle or math.random()*2*math.pi
    for i = 1, amount ,1 do
        local speed = math.random(minSpeed, maxSpeed)
        local angle = startAngle + math.random()*(deltaAngle and deltaAngle or (group and math.pi/4 or 2*math.pi))

        local x, y = cx + math.cos(angle)*600, cy + math.sin(angle)*600
        if x < 0 then x = 0 elseif x > g.getWidth() then x = g.getWidth() end
        if y < 0 then y = 0 elseif y > g.getHeight() then y = g.getHeight() end
        local dist = math.sqrt(math.pow(x - cx, 2) + math.pow(y - cy, 2))
        local newdist = dist + self.distance
        x, y = cx + (x - cx) / dist * newdist, cy + (y - cy) / dist * newdist

        kind.new(x, y, speed)

        dx, dy = dx + x, dy + y
    end

    if group then
        dx, dy = dx / amount, dy / amount
        if dx < 0 then dx = 0 elseif dx > g.getWidth() then dx = g.getWidth() end
        if dy < 0 then dy = 0 elseif dy > g.getHeight() then dy = g.getHeight() end

        local i = 1
        while self.danger[i] and self.danger[i] ~= 0 do
            i = i + 1
        end
        if i > #(self.danger) then
            table.insert(self.danger, {x = dx, y = dy, ratio = 0})
        else
            self.danger[i] =  {x = dx, y = dy, ratio = 0}
        end

        --effecting
        love.audio.newSource(content.sounds.danger):play()
        self:seq(function ()
            local life = self.danger[i].life

            tweener.tween(self.danger[i], 0.1, {ratio = 1})
            tweener.tween(self.danger[i], 0.5, {ratio = 0})
            tweener.tween(self.danger[i], 0.5, {ratio = 1})
            tweener.tween(self.danger[i], 0.5, {ratio = 0})
            tweener.tween(self.danger[i], 0.5, {ratio = 1})
            tweener.tween(self.danger[i], 0.5, {ratio = 0})
            tweener.tween(self.danger[i], 0.5, {ratio = 1})
            tweener.tween(self.danger[i], 0.5, {ratio = 0})
            tweener.tween(self.danger[i], 0.5, {ratio = 1})
            tweener.tween(self.danger[i], 0.5, {ratio = 0})

            self.danger[i] = 0
        end)
    end
end

function Spawner:addItem(kind, fallSpeed, lifeTime)
    kind.new(30 + math.random()*900, -50, fallSpeed, lifeTime)
end

function Spawner:wave(list, after)
    self:seq(function ()
        for i, v in ipairs(list) do
            if tonumber(v) then
                wait(v)
            elseif v[1] == 'item' then
                self:addItem(v[2], v[3], v[4])
            elseif v[1] == 'enemy' then
                table.remove(v, 1)
                self:addEnemy(unpack(v))
            else
                self:addEnemy(unpack(v))
            end
        end

        if after then after() end
    end)
end