require 'flatmoon.tweener'

local function roundrect(mode, x, y, width, height, xround, yround)
    local points = {}
    local precision = (xround + yround) * .1
    local tI, hP = table.insert, .5*math.pi
    if xround > width*.5 then xround = width*.5 end
    if yround > height*.5 then yround = height*.5 end
    local X1, Y1, X2, Y2 = x + xround, y + yround, x + width - xround, y + height - yround
    local sin, cos = math.sin, math.cos
    for i = 0, precision do
        local a = (i/precision-1)*hP
        tI(points, X2 + xround*cos(a))
        tI(points, Y1 + yround*sin(a))
    end
    for i = 0, precision do
        local a = (i/precision)*hP
        tI(points, X2 + xround*cos(a))
        tI(points, Y2 + yround*sin(a))
    end
    for i = 0, precision do
        local a = (i/precision+1)*hP
        tI(points, X1 + xround*cos(a))
        tI(points, Y2 + yround*sin(a))
    end
    for i = 0, precision do
        local a = (i/precision+2)*hP
        tI(points, X1 + xround*cos(a))
        tI(points, Y1 + yround*sin(a))
    end
    love.graphics.polygon(mode, unpack(points))
end

GUI = {}

local gui = {}
gui.__index = gui

local fnt_default = content.fonts.normal

function gui:draw()
    for i, b in ipairs(self.buttons) do
        g.setColor(b.colorFill)
        g.rectangle('fill', b.x, b.y, b.w, b.h)
        --roundrect('fill', b.x, b.y, b.w, b.h, 10, 10)
        g.setColor(b.colorBorder)
        g.rectangle('line', b.x, b.y, b.w, b.h)
        --roundrect('line', b.x, b.y, b.w, b.h, 10, 10)
        g.setColor(b.colorText)
        g.setFont(b.font)
        g.printf(b.text, b.x, b.y + (b.h - b.font:getHeight())/2, b.w, 'center')
    end
end

function gui:addButton(text, x, y, w, h, onClick)
    local btn = {}

    btn.text, btn.x, btn.y, btn.w, btn.h, btn.onClick = text, x, y, w, h, onClick
    btn.font = fnt_default
    btn.colorFill = {50, 50, 50, 150}
    btn.colorBorder = {150, 150, 150, 50}
    btn.colorText = {255, 255, 255, 255}

    table.insert(self.buttons, btn)

    return btn
end

function gui:always()
    local mx, my = love.mouse.getX(), love.mouse.getY()

    for i, b in ipairs(self.buttons) do
        if mx >= b.x and mx <= b.x + b.w and my >= b.y and my <= b.y + b.h and not b.inHovering then
            b.inHovering = true
            self:seq(function ()
                tweener.tween(b, 0.5, {colorFill = {200, 200, 200, 150}}, nil, nil, function () return b.inHovering end)
            end)
        elseif b.inHovering then
            b.inHovering = false
            self:seq(function ()
                tweener.tween(b, 0.5, {colorFill = {50, 50, 50, 150}}, nil, nil, function () return not b.inHovering end)
            end)
        end
    end
end

function gui:onMouseDown(mx, my)
    for i, b in ipairs(self.buttons) do
        if mx >= b.x and mx <= b.x + b.w and my >= b.y and my <= b.y + b.h then
            b.inDown = true
        end
    end
end

function gui:onMouseUp(mx, my)
    for i, b in ipairs(self.buttons) do
        if mx >= b.x and mx <= b.x + b.w and my >= b.y and my <= b.y + b.h and b.inDown then
            b.inDown = false

            if b.onClick then b:onClick() end
        end
    end
end

function GUI.new(...)
    local self = Thing.new({'GUI'})

    setmetatable(gui, getmetatable(self))
    setmetatable(self, gui)

    self.buttons = {}

    local l1 = listener.add('mousepressed', 'l', function (...) self:onMouseDown(...) end)
    local l2 = listener.add('mousereleased', 'l', function (...) self:onMouseUp(...) end)

    self.onDestroy = function (self)
        listener.remove2(l1)
        listener.remove2(l2)
    end

    return self
end